#!/bin/bash
#
# packager                          : Frederic Boulet <bipbip> bipbiponlinux~AT~gmail.com
#
# dernières révisions       : 16/01/2011
#
#package			: metamorphose
#homepage			: http://file-folder-ren.sourceforge.net/index.php
#source location	: http://downloads.sourceforge.net/file-folder-ren/metamorphose_1.1.1.tar.gz
#dependency         : python,wxPython 2.6

softname='metamorphose'
softversion='1.1.2'
packageversion='70.1'

#
# ZenWalk standard: type CPUet architecture
#
# arch='i486' -> le compileur utilise seulement les instructions 80486
# cpu='i686'  -> optimisation faîte pour PII et >
#
arch='noarch'
cpu='i686'

#
# destination du package
#
prefix=/usr

#
# log
#
(

#
# building-computer dependent: the number of parallel 'jobs' of 'make'
#
# "gros" PC: 6 compilations à la fois
#
# numjobs='-j6'
#
# "léger" PC: 2 compilations à la fois
#
numjobs='-j2'

#
# paramètres pour compiler
#
buildir=$(pwd)
src="$buildir/$softname"-"$softversion"
srcpkg="$buildir/$softname"-"$softversion.tar.gz"
package="$softname-$softversion-$arch-$packageversion"
dest="$buildir/$package"
source=http://switch.dl.sourceforge.net/sourceforge/file-folder-ren/"$softname"-$softversion.tar.gz

#
# preparer le répertoire qui accueillera la compilation
#
rm -rf $dest
mkdir -p $dest
mkdir -p $dest/install
mkdir -p $dest/$prefix/bin
mkdir -p $dest/usr/share/$softname
mkdir -p $dest/usr/share/applications
mkdir -p $dest/usr/share/icons/hicolor/16x16/apps
mkdir -p $dest/usr/share/icons/hicolor/32x32/apps
mkdir -p $dest/usr/share/icons/hicolor/48x48/apps
mkdir -p $dest/usr/share/icons/hicolor/64x64/apps
mkdir -p $dest/usr/share/icons/hicolor/128x128/apps
mkdir -p $dest/usr/doc/$softname-$softversion
mkdir -p $dest/usr/src/$softname-$softversion
mkdir -p $dest/usr/man/man1/

#
# mettre le slack-desc en place. 
#
cat <<EOF > $dest/install/slack-desc 
            |-----handy-ruler------------------------------------------------------|
metamorphose: metamorphose - a program to mass rename files and folders
metamorphose:
metamorphose: Allows many different renaming operations in a single utility, 
metamorphose: great for those of us that need to rename large numbers of files 
metamorphose: and/or folders on a regular basis.However, the ease of the interface
metamorphose: ensures a one-time user (I need to rename 5 gigs of mp3's) 
metamorphose: should not find it too frustrating to use. 
metamorphose:
metamorphose:
metamorphose:
metamorphose:
EOF

#
# Verifie si package est present
#
if [ -f "$softname"_$softversion.tar.gz ]; then
	echo "Source file already here, no need to download";
else
 	echo "Downloading source"
	wget -c --no-check-certificate $source
fi

#
# extraire les sources
#
# tar.gz : tar zxvf $srcpkg
# tar.bz2: tar xjvf $srcpkg
# tgz    : tar xzvf $srcpkg
#
cd $buildir
tar xzvf $srcpkg
cd $src	

#
# "faire" le package package
#
cp *.py $dest/usr/share/$softname
cp -R configs $dest/usr/share/$softname
cp -R icons $dest/usr/share/$softname

for lang in `ls -1 messages`; do\
    if [ -d messages/${lang} ]; then\
       mkdir -p $dest/usr/share/locale/${lang}/LC_MESSAGES;\
       cp -p messages/${lang}/LC_MESSAGES/metamorphose.mo $dest/usr/share/locale/${lang}/LC_MESSAGES/;\
    fi;\
done;\

install -m 755 metamorphose $dest/usr/bin;
install -m 755 generate_pot.sh $dest/usr/share/$softname;
   
#
# "faire" icon
#
cd $dest/usr/share/icons/hicolor/16x16/apps/
ln -s ../../../../$softname/icons/metamorphose16.png metamorphose.png
cd $dest/usr/share/icons/hicolor/32x32/apps/
ln -s ../../../../$softname/icons/metamorphose32.png metamorphose.png
cd $dest/usr/share/icons/hicolor/48x48/apps/
ln -s ../../../../$softname/icons/metamorphose48.png metamorphose.png
cd $dest/usr/share/icons/hicolor/64x64/apps/
ln -s ../../../../$softname/icons/metamorphose64.png metamorphose.png
cd $dest/usr/share/icons/hicolor/128x128/apps/
ln -s ../../../../$softname/icons/metamorphose128.png metamorphose.png

# "faire" .desktop
cat << "EOF" > $dest/usr/share/applications/$softname.desktop
[Desktop Entry]
Encoding=UTF-8
Name=Metamorphose
Comment=A mass renamer for files and folders
Comment[de]=Ein Programm um Dateien und Ordner umzubenennen
Comment[es]=Un renombrador masivo para archivos y carpetas
Comment[fr]=Un renommeur massif pour fichiers et dossiers
Exec=metamorphose
Terminal=false
Type=Application
Categories=Application;Utility;
Icon=metamorphose
EOF

#
# ajout des fichiers de docs, infos, ...
#
cd $src/docs
cp -R \
	  hu es it de zh_CN ja en_US ru pl ar fr tr he pt_BR\
      $dest/usr/doc/$softname-$softversion
cd $src	
cp *.html $dest/usr/doc/$softname-$softversion
cp License.txt $dest/usr/doc/$softname-$softversion
cp metamorphose.1 $dest/usr/man/man1/
	
#
# Compresser les pages Man
#

( cd $dest/$prefix/man
  find . -name "*.?" | xargs gzip -9
)

#
# Stripper les éxécutables
# (Commenter ces lignes si besoin d'infos de débuggage)
#
( cd $dest
  find . | xargs file | grep "executable" | grep ELF | cut -f 1 -d : | xargs strip --strip-unneeded 2> /dev/null
  find . | xargs file | grep "shared object" | grep ELF | cut -f 1 -d : | xargs strip --strip-unneeded 2> /dev/null
)

#
# Mettre le zenbuild à sa place
#
cd $buildir
cp build-$softname.sh $dest/$prefix/src/$softname-$softversion

#
# fixer les permissions
#
chown -R root.root $dest

cd $dest
find . -perm 664 -exec chmod 644 {} \;
find . -perm 600 -exec chmod 644 {} \;
find . -perm 444 -exec chmod 644 {} \;
find . -perm 400 -exec chmod 644 {} \;
find . -perm 440 -exec chmod 644 {} \;
find . -perm 777 -exec chmod 755 {} \;
find . -perm 775 -exec chmod 755 {} \;
find . -perm 511 -exec chmod 755 {} \;
find . -perm 711 -exec chmod 755 {} \;
find . -perm 555 -exec chmod 755 {} \;

#
# fabriquer le zenpackage
#
cd $dest
makepkg -l y -c n $dest.txz

#
# produire la signature md5 du package
#
cd $buildir
md5sum $package.txz > $package.md5

#
# produire le fichier .dep
#
cd $buildir
echo -e "bzip2,gdbm,ncurses,openssl,python,readline,wxpython,xorg-libs,zlib" > $package.dep

#
# supprimer source
#
rm -rf $src
rm -rf $dest

#
# genere fichier source
#
echo http://zenwalk.pinguix.com/user-accounts/bip/EXTRA/xap/metamorphose/source/build-metamorphose.sh > $package.src
echo $source >>$package.src

#
# log
#
) 2>&1 | tee "$softname-$softversion-build.log"
